@@ -2,8 +2,12 @@
 
 from __future__ import annotations
 
+import io
+import asyncio
+import tarfile
 from typing import Dict, Mapping, Optional
 from pathlib import Path
+from datetime import timedelta
 from typing_extensions import Unpack
 
 import httpx
@@ -17,7 +21,7 @@ from ._types import (
     SDKBlueprintListParams,
     SDKBlueprintCreateParams,
     SDKDiskSnapshotListParams,
-    SDKDevboxExtraCreateParams,
+    SDKDevboxCreateFromImageParams,
 )
 from .._types import Timeout, NotGiven, not_given
 from .._client import DEFAULT_MAX_RETRIES, AsyncRunloop
@@ -45,8 +49,8 @@ class AsyncDevboxOps:
     def __init__(self, client: AsyncRunloop) -> None:
         """Initialize the manager.
 
-        Args:
-            client: Generated AsyncRunloop client to wrap.
+        :param client: Generated AsyncRunloop client to wrap
+        :type client: AsyncRunloop
         """
         self._client = client
 
@@ -56,11 +60,9 @@ class AsyncDevboxOps:
     ) -> AsyncDevbox:
         """Provision a new devbox and wait until it reaches ``running`` state.
 
-        Args:
-            **params: Keyword arguments forwarded to the devbox creation API.
-
-        Returns:
-            AsyncDevbox: Wrapper bound to the newly created devbox.
+        :param params: See :typeddict:`~runloop_api_client.sdk._types.SDKDevboxCreateParams` for available parameters
+        :return: Wrapper bound to the newly created devbox
+        :rtype: AsyncDevbox
         """
         devbox_view = await self._client.devboxes.create_and_await_running(
             **params,
@@ -70,16 +72,15 @@ class AsyncDevboxOps:
     async def create_from_blueprint_id(
         self,
         blueprint_id: str,
-        **params: Unpack[SDKDevboxExtraCreateParams],
+        **params: Unpack[SDKDevboxCreateFromImageParams],
     ) -> AsyncDevbox:
         """Create a devbox from an existing blueprint by identifier.
 
-        Args:
-            blueprint_id: Blueprint ID to create from.
-            **params: Additional creation parameters (metadata, launch parameters, etc.).
-
-        Returns:
-            AsyncDevbox: Wrapper bound to the newly created devbox.
+        :param blueprint_id: Blueprint ID to create from
+        :type blueprint_id: str
+        :param params: See :typeddict:`~runloop_api_client.sdk._types.SDKDevboxCreateFromImageParams` for available parameters
+        :return: Wrapper bound to the newly created devbox
+        :rtype: AsyncDevbox
         """
         devbox_view = await self._client.devboxes.create_and_await_running(
             blueprint_id=blueprint_id,
@@ -90,16 +91,15 @@ class AsyncDevboxOps:
     async def create_from_blueprint_name(
         self,
         blueprint_name: str,
-        **params: Unpack[SDKDevboxExtraCreateParams],
+        **params: Unpack[SDKDevboxCreateFromImageParams],
     ) -> AsyncDevbox:
         """Create a devbox from the latest blueprint with the given name.
 
-        Args:
-            blueprint_name: Blueprint name to create from.
-            **params: Additional creation parameters (metadata, launch parameters, etc.).
-
-        Returns:
-            AsyncDevbox: Wrapper bound to the newly created devbox.
+        :param blueprint_name: Blueprint name to create from
+        :type blueprint_name: str
+        :param params: See :typeddict:`~runloop_api_client.sdk._types.SDKDevboxCreateFromImageParams` for available parameters
+        :return: Wrapper bound to the newly created devbox
+        :rtype: AsyncDevbox
         """
         devbox_view = await self._client.devboxes.create_and_await_running(
             blueprint_name=blueprint_name,
@@ -110,16 +110,15 @@ class AsyncDevboxOps:
     async def create_from_snapshot(
         self,
         snapshot_id: str,
-        **params: Unpack[SDKDevboxExtraCreateParams],
+        **params: Unpack[SDKDevboxCreateFromImageParams],
     ) -> AsyncDevbox:
         """Create a devbox initialized from a snapshot.
 
-        Args:
-            snapshot_id: Snapshot ID to create from.
-            **params: Additional creation parameters (metadata, launch parameters, etc.).
-
-        Returns:
-            AsyncDevbox: Wrapper bound to the newly created devbox.
+        :param snapshot_id: Snapshot ID to create from
+        :type snapshot_id: str
+        :param params: See :typeddict:`~runloop_api_client.sdk._types.SDKDevboxCreateFromImageParams` for available parameters
+        :return: Wrapper bound to the newly created devbox
+        :rtype: AsyncDevbox
         """
         devbox_view = await self._client.devboxes.create_and_await_running(
             snapshot_id=snapshot_id,
@@ -130,11 +129,14 @@ class AsyncDevboxOps:
     def from_id(self, devbox_id: str) -> AsyncDevbox:
         """Attach to an existing devbox by ID.
 
-        Args:
-            devbox_id: Existing devbox ID.
+        Returns immediately without waiting for the devbox to reach ``running``
+        state. Call ``await_running()`` on the returned :class:`AsyncDevbox` if
+        you need to wait for readiness (contrast with the synchronous SDK, which blocks).
 
-        Returns:
-            AsyncDevbox: Wrapper bound to the requested devbox.
+        :param devbox_id: Existing devbox ID
+        :type devbox_id: str
+        :return: Wrapper bound to the requested devbox
+        :rtype: AsyncDevbox
         """
         return AsyncDevbox(self._client, devbox_id)
 
@@ -144,11 +146,9 @@ class AsyncDevboxOps:
     ) -> list[AsyncDevbox]:
         """List devboxes accessible to the caller.
 
-        Args:
-            **params: Filtering and pagination parameters.
-
-        Returns:
-            list[AsyncDevbox]: Collection of devbox wrappers.
+        :param params: See :typeddict:`~runloop_api_client.sdk._types.SDKDevboxListParams` for available parameters
+        :return: Collection of devbox wrappers
+        :rtype: list[AsyncDevbox]
         """
         page = await self._client.devboxes.list(
             **params,
@@ -171,8 +171,8 @@ class AsyncSnapshotOps:
     def __init__(self, client: AsyncRunloop) -> None:
         """Initialize the manager.
 
-        Args:
-            client: Generated AsyncRunloop client to wrap.
+        :param client: Generated AsyncRunloop client to wrap
+        :type client: AsyncRunloop
         """
         self._client = client
 
@@ -182,11 +182,9 @@ class AsyncSnapshotOps:
     ) -> list[AsyncSnapshot]:
         """List snapshots created from devboxes.
 
-        Args:
-            **params: Filtering and pagination parameters.
-
-        Returns:
-            list[AsyncSnapshot]: Snapshot wrappers for each record.
+        :param params: See :typeddict:`~runloop_api_client.sdk._types.SDKDiskSnapshotListParams` for available parameters
+        :return: Snapshot wrappers for each record
+        :rtype: list[AsyncSnapshot]
         """
         page = await self._client.devboxes.disk_snapshots.list(
             **params,
@@ -196,11 +194,10 @@ class AsyncSnapshotOps:
     def from_id(self, snapshot_id: str) -> AsyncSnapshot:
         """Return a snapshot wrapper for the given ID.
 
-        Args:
-            snapshot_id: Snapshot ID to wrap.
-
-        Returns:
-            AsyncSnapshot: Wrapper for the snapshot resource.
+        :param snapshot_id: Snapshot ID to wrap
+        :type snapshot_id: str
+        :return: Wrapper for the snapshot resource
+        :rtype: AsyncSnapshot
         """
         return AsyncSnapshot(self._client, snapshot_id)
 
@@ -224,8 +221,8 @@ class AsyncBlueprintOps:
     def __init__(self, client: AsyncRunloop) -> None:
         """Initialize the manager.
 
-        Args:
-            client: Generated AsyncRunloop client to wrap.
+        :param client: Generated AsyncRunloop client to wrap
+        :type client: AsyncRunloop
         """
         self._client = client
 
@@ -235,11 +232,9 @@ class AsyncBlueprintOps:
     ) -> AsyncBlueprint:
         """Create a blueprint and wait for the build to finish.
 
-        Args:
-            **params: Blueprint definition (Dockerfile, metadata, etc.).
-
-        Returns:
-            AsyncBlueprint: Wrapper bound to the finished blueprint.
+        :param params: See :typeddict:`~runloop_api_client.sdk._types.SDKBlueprintCreateParams` for available parameters
+        :return: Wrapper bound to the finished blueprint
+        :rtype: AsyncBlueprint
         """
         blueprint = await self._client.blueprints.create_and_await_build_complete(
             **params,
@@ -249,11 +244,10 @@ class AsyncBlueprintOps:
     def from_id(self, blueprint_id: str) -> AsyncBlueprint:
         """Return a blueprint wrapper for the given ID.
 
-        Args:
-            blueprint_id: Blueprint ID to wrap.
-
-        Returns:
-            AsyncBlueprint: Wrapper for the blueprint resource.
+        :param blueprint_id: Blueprint ID to wrap
+        :type blueprint_id: str
+        :return: Wrapper for the blueprint resource
+        :rtype: AsyncBlueprint
         """
         return AsyncBlueprint(self._client, blueprint_id)
 
@@ -263,11 +257,9 @@ class AsyncBlueprintOps:
     ) -> list[AsyncBlueprint]:
         """List available blueprints.
 
-        Args:
-            **params: Filtering and pagination parameters.
-
-        Returns:
-            list[AsyncBlueprint]: Blueprint wrappers for each record.
+        :param params: See :typeddict:`~runloop_api_client.sdk._types.SDKBlueprintListParams` for available parameters
+        :return: Blueprint wrappers for each record
+        :rtype: list[AsyncBlueprint]
         """
         page = await self._client.blueprints.list(
             **params,
@@ -292,8 +284,8 @@ class AsyncStorageObjectOps:
     def __init__(self, client: AsyncRunloop) -> None:
         """Initialize the manager.
 
-        Args:
-            client: Generated AsyncRunloop client to wrap.
+        :param client: Generated AsyncRunloop client to wrap
+        :type client: AsyncRunloop
         """
         self._client = client
 
@@ -303,11 +295,9 @@ class AsyncStorageObjectOps:
     ) -> AsyncStorageObject:
         """Create a storage object and obtain an upload URL.
 
-        Args:
-            **params: Object creation parameters (name, content type, metadata).
-
-        Returns:
-            AsyncStorageObject: Wrapper with upload URL set for immediate uploads.
+        :param params: See :typeddict:`~runloop_api_client.sdk._types.SDKObjectCreateParams` for available parameters
+        :return: Wrapper with upload URL set for immediate uploads
+        :rtype: AsyncStorageObject
         """
         obj = await self._client.objects.create(**params)
         return AsyncStorageObject(self._client, obj.id, upload_url=obj.upload_url)
@@ -315,11 +305,10 @@ class AsyncStorageObjectOps:
     def from_id(self, object_id: str) -> AsyncStorageObject:
         """Return a storage object wrapper by identifier.
 
-        Args:
-            object_id: Storage object identifier to wrap.
-
-        Returns:
-            AsyncStorageObject: Wrapper for the storage object resource.
+        :param object_id: Storage object identifier to wrap
+        :type object_id: str
+        :return: Wrapper for the storage object resource
+        :rtype: AsyncStorageObject
         """
         return AsyncStorageObject(self._client, object_id, upload_url=None)
 
@@ -329,11 +318,9 @@ class AsyncStorageObjectOps:
     ) -> list[AsyncStorageObject]:
         """List storage objects owned by the caller.
 
-        Args:
-            **params: Filtering and pagination parameters.
-
-        Returns:
-            list[AsyncStorageObject]: Storage object wrappers for each record.
+        :param params: See :typeddict:`~runloop_api_client.sdk._types.SDKObjectListParams` for available parameters
+        :return: Storage object wrappers for each record
+        :rtype: list[AsyncStorageObject]
         """
         page = await self._client.objects.list(
             **params,
@@ -351,23 +338,23 @@ class AsyncStorageObjectOps:
     ) -> AsyncStorageObject:
         """Create and upload an object from a local file path.
 
-        Args:
-            file_path: Local filesystem path to read.
-            name: Optional object name; defaults to the file name.
-            content_type: Optional MIME type to apply to the object.
-            metadata: Optional key-value metadata.
-            **options: Additional request configuration.
-
-        Returns:
-            AsyncStorageObject: Wrapper for the uploaded object.
-
-        Raises:
-            OSError: If the local file cannot be read.
+        :param file_path: Local filesystem path to read
+        :type file_path: str | Path
+        :param name: Optional object name; defaults to the file name, defaults to None
+        :type name: str | None, optional
+        :param content_type: Optional MIME type to apply to the object, defaults to None
+        :type content_type: ContentType | None, optional
+        :param metadata: Optional key-value metadata, defaults to None
+        :type metadata: Optional[Dict[str, str]], optional
+        :param options: See :typeddict:`~runloop_api_client.sdk._types.LongRequestOptions` for available options
+        :return: Wrapper for the uploaded object
+        :rtype: AsyncStorageObject
+        :raises OSError: If the local file cannot be read
         """
         path = Path(file_path)
 
         try:
-            content = path.read_bytes()
+            content = await asyncio.to_thread(lambda: path.read_bytes())
         except OSError as error:
             raise OSError(f"Failed to read file {path}: {error}") from error
 
@@ -378,6 +365,50 @@ class AsyncStorageObjectOps:
         await obj.complete()
         return obj
 
+    async def upload_from_dir(
+        self,
+        dir_path: str | Path,
+        *,
+        name: Optional[str] = None,
+        metadata: Optional[Dict[str, str]] = None,
+        ttl: Optional[timedelta] = None,
+        **options: Unpack[LongRequestOptions],
+    ) -> AsyncStorageObject:
+        """Create and upload an object from a local directory.
+
+        The resulting object will be uploaded as a compressed tarball.
+
+        :param dir_path: Local filesystem directory path to tar
+        :type dir_path: str | Path
+        :param name: Optional object name; defaults to the directory name + '.tar.gz'
+        :type name: Optional[str]
+        :param metadata: Optional key-value metadata
+        :type metadata: Optional[Dict[str, str]]
+        :param ttl: Optional Time-To-Live, after which the object is automatically deleted
+        :type ttl: Optional[timedelta]
+        :param options: See :typeddict:`~runloop_api_client.sdk._types.LongRequestOptions` for available options
+        :return: Wrapper for the uploaded object
+        :rtype: AsyncStorageObject
+        :raises OSError: If the local file cannot be read
+        """
+        path = Path(dir_path)
+        name = name or f"{path.name}.tar.gz"
+        ttl_ms = int(ttl.total_seconds()) * 1000 if ttl else None
+
+        def synchronous_io() -> bytes:
+            with io.BytesIO() as tar_buffer:
+                with tarfile.open(fileobj=tar_buffer, mode="w:gz") as tar:
+                    tar.add(path, arcname=".", recursive=True)
+                tar_buffer.seek(0)
+                return tar_buffer.read()
+
+        tar_bytes = await asyncio.to_thread(synchronous_io)
+
+        obj = await self.create(name=name, content_type="tgz", metadata=metadata, ttl_ms=ttl_ms, **options)
+        await obj.upload_content(tar_bytes)
+        await obj.complete()
+        return obj
+
     async def upload_from_text(
         self,
         text: str,
@@ -388,14 +419,15 @@ class AsyncStorageObjectOps:
     ) -> AsyncStorageObject:
         """Create and upload an object from a text payload.
 
-        Args:
-            text: Text content to upload.
-            name: Object display name.
-            metadata: Optional key-value metadata.
-            **options: Additional request configuration.
-
-        Returns:
-            AsyncStorageObject: Wrapper for the uploaded object.
+        :param text: Text content to upload
+        :type text: str
+        :param name: Object display name
+        :type name: str
+        :param metadata: Optional key-value metadata, defaults to None
+        :type metadata: Optional[Dict[str, str]], optional
+        :param options: See :typeddict:`~runloop_api_client.sdk._types.LongRequestOptions` for available options
+        :return: Wrapper for the uploaded object
+        :rtype: AsyncStorageObject
         """
         obj = await self.create(name=name, content_type="text", metadata=metadata, **options)
         await obj.upload_content(text)
@@ -413,15 +445,17 @@ class AsyncStorageObjectOps:
     ) -> AsyncStorageObject:
         """Create and upload an object from a bytes payload.
 
-        Args:
-            data: Binary payload to upload.
-            name: Object display name.
-            content_type: MIME type describing the payload.
-            metadata: Optional key-value metadata.
-            **options: Additional request configuration.
-
-        Returns:
-            AsyncStorageObject: Wrapper for the uploaded object.
+        :param data: Binary payload to upload
+        :type data: bytes
+        :param name: Object display name
+        :type name: str
+        :param content_type: MIME type describing the payload
+        :type content_type: ContentType
+        :param metadata: Optional key-value metadata, defaults to None
+        :type metadata: Optional[Dict[str, str]], optional
+        :param options: See :typeddict:`~runloop_api_client.sdk._types.LongRequestOptions` for available options
+        :return: Wrapper for the uploaded object
+        :rtype: AsyncStorageObject
         """
         obj = await self.create(name=name, content_type=content_type, metadata=metadata, **options)
         await obj.upload_content(data)
@@ -436,12 +470,16 @@ class AsyncRunloopSDK:
     blueprints, snapshots, and storage objects. Exposes the generated async REST
     client via the ``api`` attribute for advanced use cases.
 
-    Attributes:
-        api: Direct access to the generated async REST API client.
-        devbox: High-level async interface for devbox management.
-        blueprint: High-level async interface for blueprint management.
-        snapshot: High-level async interface for snapshot management.
-        storage_object: High-level async interface for storage object management.
+    :ivar api: Direct access to the generated async REST API client
+    :vartype api: AsyncRunloop
+    :ivar devbox: High-level async interface for devbox management
+    :vartype devbox: AsyncDevboxOps
+    :ivar blueprint: High-level async interface for blueprint management
+    :vartype blueprint: AsyncBlueprintOps
+    :ivar snapshot: High-level async interface for snapshot management
+    :vartype snapshot: AsyncSnapshotOps
+    :ivar storage_object: High-level async interface for storage object management
+    :vartype storage_object: AsyncStorageObjectOps
 
     Example:
         >>> runloop = AsyncRunloopSDK()  # Uses RUNLOOP_API_KEY env var
@@ -470,14 +508,20 @@ class AsyncRunloopSDK:
     ) -> None:
         """Configure the asynchronous SDK wrapper.
 
-        Args:
-            bearer_token: API token; falls back to ``RUNLOOP_API_KEY`` env var.
-            base_url: Override the API base URL.
-            timeout: Request timeout (seconds) or ``Timeout`` object.
-            max_retries: Maximum automatic retry attempts.
-            default_headers: Headers merged into every request.
-            default_query: Default query parameters merged into every request.
-            http_client: Custom ``httpx.AsyncClient`` instance to reuse.
+        :param bearer_token: API token; falls back to ``RUNLOOP_API_KEY`` env var, defaults to None
+        :type bearer_token: str | None, optional
+        :param base_url: Override the API base URL, defaults to None
+        :type base_url: str | httpx.URL | None, optional
+        :param timeout: Request timeout (seconds) or ``Timeout`` object, defaults to not_given
+        :type timeout: float | Timeout | None | NotGiven, optional
+        :param max_retries: Maximum automatic retry attempts, defaults to DEFAULT_MAX_RETRIES
+        :type max_retries: int, optional
+        :param default_headers: Headers merged into every request, defaults to None
+        :type default_headers: Mapping[str, str] | None, optional
+        :param default_query: Default query parameters merged into every request, defaults to None
+        :type default_query: Mapping[str, object] | None, optional
+        :param http_client: Custom ``httpx.AsyncClient`` instance to reuse, defaults to None
+        :type http_client: httpx.AsyncClient | None, optional
         """
         self.api = AsyncRunloop(
             bearer_token=bearer_token,
@@ -501,8 +545,8 @@ class AsyncRunloopSDK:
     async def __aenter__(self) -> "AsyncRunloopSDK":
         """Allow ``async with AsyncRunloopSDK() as runloop`` usage.
 
-        Returns:
-            AsyncRunloopSDK: The active SDK instance.
+        :return: The active SDK instance
+        :rtype: AsyncRunloopSDK
         """
         return self
 
