@@ -2,6 +2,8 @@
 
 from __future__ import annotations
 
+import io
+import tarfile
 from types import SimpleNamespace
 from pathlib import Path
 from unittest.mock import AsyncMock
@@ -327,6 +329,161 @@ class TestAsyncStorageObjectClient:
         with pytest.raises(OSError, match="Failed to read file"):
             await client.upload_from_file(missing_file)
 
+    @pytest.mark.asyncio
+    async def test_upload_from_dir(
+        self, mock_async_client: AsyncMock, object_view: MockObjectView, tmp_path: Path
+    ) -> None:
+        """Test upload_from_dir method."""
+        mock_async_client.objects.create = AsyncMock(return_value=object_view)
+        mock_async_client.objects.complete = AsyncMock(return_value=object_view)
+
+        # Create a temporary directory with some files
+        test_dir = tmp_path / "test_directory"
+        test_dir.mkdir()
+        (test_dir / "file1.txt").write_text("content1")
+        (test_dir / "file2.txt").write_text("content2")
+        subdir = test_dir / "subdir"
+        subdir.mkdir()
+        (subdir / "file3.txt").write_text("content3")
+
+        http_client = AsyncMock()
+        mock_response = create_mock_httpx_response()
+        http_client.put = AsyncMock(return_value=mock_response)
+        mock_async_client._client = http_client
+
+        client = AsyncStorageObjectOps(mock_async_client)
+        obj = await client.upload_from_dir(test_dir, name="archive.tar.gz", metadata={"key": "value"})
+
+        assert isinstance(obj, AsyncStorageObject)
+        assert obj.id == "obj_123"
+        mock_async_client.objects.create.assert_awaited_once_with(
+            name="archive.tar.gz",
+            content_type="tgz",
+            metadata={"key": "value"},
+            ttl_ms=None,
+        )
+        # Verify that put was called with tarball content
+        http_client.put.assert_awaited_once()
+        call_args = http_client.put.call_args
+        assert call_args[0][0] == object_view.upload_url
+
+        # Verify it's a valid gzipped tarball
+        uploaded_content = call_args[1]["content"]
+        with tarfile.open(fileobj=io.BytesIO(uploaded_content), mode="r:gz") as tar:
+            members = tar.getmembers()
+            member_names = [m.name for m in members]
+            # Should contain our test files (may include directory entries)
+            assert any("file1.txt" in name for name in member_names)
+            assert any("file2.txt" in name for name in member_names)
+            assert any("file3.txt" in name for name in member_names)
+
+        mock_async_client.objects.complete.assert_awaited_once()
+
+    @pytest.mark.asyncio
+    async def test_upload_from_dir_default_name(
+        self, mock_async_client: AsyncMock, object_view: MockObjectView, tmp_path: Path
+    ) -> None:
+        """Test upload_from_dir uses directory name by default."""
+        mock_async_client.objects.create = AsyncMock(return_value=object_view)
+        mock_async_client.objects.complete = AsyncMock(return_value=object_view)
+
+        test_dir = tmp_path / "my_folder"
+        test_dir.mkdir()
+        (test_dir / "file.txt").write_text("content")
+
+        http_client = AsyncMock()
+        mock_response = create_mock_httpx_response()
+        http_client.put = AsyncMock(return_value=mock_response)
+        mock_async_client._client = http_client
+
+        client = AsyncStorageObjectOps(mock_async_client)
+        obj = await client.upload_from_dir(test_dir)
+
+        assert isinstance(obj, AsyncStorageObject)
+        # Name should be directory name + .tar.gz
+        mock_async_client.objects.create.assert_awaited_once()
+        call_args = mock_async_client.objects.create.call_args
+        assert call_args[1]["name"] == "my_folder.tar.gz"
+        assert call_args[1]["content_type"] == "tgz"
+
+    @pytest.mark.asyncio
+    async def test_upload_from_dir_with_ttl(
+        self, mock_async_client: AsyncMock, object_view: MockObjectView, tmp_path: Path
+    ) -> None:
+        """Test upload_from_dir with TTL."""
+        from datetime import timedelta
+
+        mock_async_client.objects.create = AsyncMock(return_value=object_view)
+        mock_async_client.objects.complete = AsyncMock(return_value=object_view)
+
+        test_dir = tmp_path / "temp_dir"
+        test_dir.mkdir()
+        (test_dir / "file.txt").write_text("temporary content")
+
+        http_client = AsyncMock()
+        mock_response = create_mock_httpx_response()
+        http_client.put = AsyncMock(return_value=mock_response)
+        mock_async_client._client = http_client
+
+        client = AsyncStorageObjectOps(mock_async_client)
+        obj = await client.upload_from_dir(test_dir, ttl=timedelta(hours=2))
+
+        assert isinstance(obj, AsyncStorageObject)
+        mock_async_client.objects.create.assert_awaited_once()
+        call_args = mock_async_client.objects.create.call_args
+        # 2 hours = 7200 seconds = 7200000 milliseconds
+        assert call_args[1]["ttl_ms"] == 7200000
+
+    @pytest.mark.asyncio
+    async def test_upload_from_dir_empty_directory(
+        self, mock_async_client: AsyncMock, object_view: MockObjectView, tmp_path: Path
+    ) -> None:
+        """Test upload_from_dir with empty directory."""
+        mock_async_client.objects.create = AsyncMock(return_value=object_view)
+        mock_async_client.objects.complete = AsyncMock(return_value=object_view)
+
+        test_dir = tmp_path / "empty_dir"
+        test_dir.mkdir()
+
+        http_client = AsyncMock()
+        mock_response = create_mock_httpx_response()
+        http_client.put = AsyncMock(return_value=mock_response)
+        mock_async_client._client = http_client
+
+        client = AsyncStorageObjectOps(mock_async_client)
+        obj = await client.upload_from_dir(test_dir)
+
+        assert isinstance(obj, AsyncStorageObject)
+        assert obj.id == "obj_123"
+        mock_async_client.objects.create.assert_awaited_once()
+        http_client.put.assert_awaited_once()
+        mock_async_client.objects.complete.assert_awaited_once()
+
+    @pytest.mark.asyncio
+    async def test_upload_from_dir_with_string_path(
+        self, mock_async_client: AsyncMock, object_view: MockObjectView, tmp_path: Path
+    ) -> None:
+        """Test upload_from_dir with string path instead of Path object."""
+        mock_async_client.objects.create = AsyncMock(return_value=object_view)
+        mock_async_client.objects.complete = AsyncMock(return_value=object_view)
+
+        test_dir = tmp_path / "string_path_dir"
+        test_dir.mkdir()
+        (test_dir / "file.txt").write_text("content")
+
+        http_client = AsyncMock()
+        mock_response = create_mock_httpx_response()
+        http_client.put = AsyncMock(return_value=mock_response)
+        mock_async_client._client = http_client
+
+        client = AsyncStorageObjectOps(mock_async_client)
+        # Pass string path instead of Path object
+        obj = await client.upload_from_dir(str(test_dir))
+
+        assert isinstance(obj, AsyncStorageObject)
+        assert obj.id == "obj_123"
+        mock_async_client.objects.create.assert_awaited_once()
+
 
 class TestAsyncRunloopSDK:
     """Tests for AsyncRunloopSDK class."""
